<#
.SYNOPSIS 
    Configures Exchange 2013 Multi-Tenant Transport Agent. 
.DESCRIPTION 
    Configures Exchange 2013 Multi-Tenant Transport Agent.
.PARAMETER Install
    Installs Transport Agent.
.PARAMETER Uninstall
    Uninstalls Transport Agent.
.PARAMETER Update
    Updates Transport Agent.
.PARAMETER PerfMonCounters
    Removes existing counters and installs new Performance Counters.
.EXAMPLE 
    Configure-TransportAgent.ps1 -Install
.EXAMPLE 
    Configure-TransportAgent.ps1 -Uninstall
.EXAMPLE 
    Configure-TransportAgent.ps1 -Update
.EXAMPLE 
    Configure-TransportAgent.ps1 -PerfMonCounters
.LINK
    Links
.NOTES 
    NAME       :  Configure-TransportAgent.ps1
    AUTHOR     :  RickettsCorp
    VERSION    :  1.6
    LASTEDIT   :  11/21/2016
#Requires -Version 2.0 
#>

[CmdletBinding()]
Param(
    [Parameter(Position=0, Mandatory=$true, ValueFromPipeline=$false)]
    [ValidateSet("Install", "Uninstall", "Update", "PerfMonCounters")]
    [string] $ScriptMode
 )

###############################################################################
# Variables
###############################################################################

$ConfigFile = "MultiTenantHubTransportAgentConfig.xml"
$ExDir = (Get-ItemProperty -Path HKLM:\Software\Microsoft\ExchangeServer\v14\Setup).MsiInstallPath

# Get OS Version
$OSVersion = [System.Environment]::OSVersion.Version
$Win2012R2BaseVersion = [Version]"6.3.0.0"

###############################################################################
#
# Function Main
#
# PURPOSE
#    Main entry point to script
#
# INPUT
#    None.
#
# RETURN
#    None.
#
###############################################################################
function Main
{   
   $TransportAgentDll = Get-ChildItem MultiTenantHubTransportAgent.dll
   
   Load-XmlConfiguration
   
   Switch($ScriptMode.ToUpper()) {    
        "INSTALL"         {Install-Agent}
        "UNINSTALL"       {Uninstall-Agent}
        "UPDATE"          {Update-Agent}
        "PERFMONCOUNTERS" {Create-PerformanceCounters}
        default           {"`nIncorrect Mode`n"}    
    }
}

###############################################################################
#
# Function Install-Agent
#
# PURPOSE
#    Installs the Transport Agent
#
# INPUT
#    None.
#
# RETURN
#    None.
#
###############################################################################
function Install-Agent
{
    $result = Display-Eula
    
    if($result -ne "Yes")
    {    
        "Eula not accepted"
        Exit
    }
    
    Write-EventLogEntry "Starting Agent Install (Version $($TransportAgentDll.VersionInfo.ProductVersion))" "Information"

    if(-not (Test-Path "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent"))
    {
        Write-Output "Creating Agent Directory"
        New-Item -Type Directory -path "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -ErrorAction SilentlyContinue | Out-Null
	Add-FolderAcl "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent"
    }

    if(-not (Test-Path $Script:LogDirectory))
    {
        Write-Output "Creating Log Directory"
        New-Item -Type Directory -path $Script:LogDirectory -ErrorAction SilentlyContinue | Out-Null
		Add-FolderAcl "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent"
    }

    Write-Output "Copying Agent Files"
    Copy-Item MultiTenantHubTransportAgent.dll "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force
    Copy-Item MultiTenantHubTransportAgent.pdb "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force
    Copy-Item MultiTenantHubTransportAgentConfig.xml "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force
    Copy-Item NLog.dll "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force

    Write-Output "Creating Performance Counters"
    Create-PerformanceCounters

    Write-Output "Registering Agent"
    Install-TransportAgent -Name "MultiTenantHubTransportAgent" -AssemblyPath "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent\MultiTenantHubTransportAgent.dll" -TransportAgentFactory RickettsCorp.Exchange.TransportAgents.MultiTenantHubTransportAgentFactory

    Write-Output "Enabling Agent"
    Enable-TransportAgent -Identity "MultiTenantHubTransportAgent"
    Get-TransportAgent -Identity "MultiTenantHubTransportAgent"
    Restart-Service MsExchangeTransport
    
}

###############################################################################
#
# Function Uninstall-Agent
#
# PURPOSE
#    Uninstalls the Transport Agent
#
# INPUT
#    None.
#
# RETURN
#    None.
#
###############################################################################
function Uninstall-Agent
{
   Write-EventLogEntry "Uninstalling Agent ($($TransportAgentDll.VersionInfo.ProductVersion))" "Information"
   
   Write-Output "Removing Agent from Transport"
   Disable-TransportAgent -Identity "MultiTenantHubTransportAgent" -Confirm:$false
   Uninstall-TransportAgent -Identity "MultiTenantHubTransportAgent" -Confirm:$false

   Write-Output "Restarting MSExchangeTransport and W3SVC Services"
   Restart-Service MSExchangeTransport
   Restart-Service W3SVC
   Get-TransportAgent
}

###############################################################################
#
# Function Update-Agent
#
# PURPOSE
#    Updates the Transport Agent
#
# INPUT
#    None.
#
# RETURN
#    None.
#
###############################################################################
function Update-Agent
{
    Write-EventLogEntry "Updating Agent Version ($($TransportAgentDll.VersionInfo.ProductVersion))" "Information"

    Write-Output "Stopping MSExchangeTransport and W3SVC services"
    Stop-Service MSExchangeTransport
    Stop-Service W3SVC -Force
    
    Write-Output "Copying Updated Agent Files (ensure config file has correct settings)"
    Copy-Item MultiTenantHubTransportAgent.dll "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force
    Copy-Item MultiTenantHubTransportAgent.pdb "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force
    Copy-Item MultiTenantHubTransportAgentConfig.xml "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force
    Copy-Item NLog.dll "$($ExDir)agents\RoutingAgents\MultiTenantHubTransportAgent" -force

    Write-Output "Restarting MSExchangeTransport and W3SVC services"
    Start-Service MSExchangeTransport
    Start-Service W3SVC

    if($OSVersion -gt $Win2012R2BaseVersion)
    {
        Start-Service W3LogSvc
    }
}

###############################################################################
#
# Function Add-FolderAcl
#
# PURPOSE
#    Adds "Modify" permission to specified folder for NETWORK SERVICE account
#
# INPUT
#    Folder Name.
#
# RETURN
#    None.
#
###############################################################################
function Add-FolderAcl($FolderName)
{
    $FolderAcl  = Get-Acl $FolderName 
    $Permission = "NETWORK SERVICE", "Modify", "ObjectInherit", "None", "Allow" 
    $AccessRule = New-Object System.Security.AccessControl.FileSystemAccessRule $Permission 
    $FolderAcl.AddAccessRule($AccessRule) 
    Set-Acl $FolderName $FolderAcl
}

###############################################################################
#
# Function Create-PerformanceCounters
#
# PURPOSE
#    Creates the Transport Agent Perfmon Category and Counters
#
# INPUT
#    None.
#
# RETURN
#    None.
#
###############################################################################
function Create-PerformanceCounters
{
    $categoryName = "Hub Transport Multi-Tenant Transport Agent"
 
    $exists = [System.Diagnostics.PerformanceCounterCategory]::Exists($categoryName)
    if ($exists)
    {
        [System.Diagnostics.PerformanceCounterCategory]::Delete($categoryName)
    }
 
    $counterData = new-object System.Diagnostics.CounterCreationDataCollection
 
    $counter1 = new-object System.Diagnostics.CounterCreationData
    $counter1.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter1.CounterName = "Total Number of Internal OOFs Analyzed"
    [Void]$counterData.Add($counter1)

    $counter2 = new-object System.Diagnostics.CounterCreationData
    $counter2.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter2.CounterName = "Total Number of Internal OOFs Deleted"
    [Void]$counterData.Add($counter2)

    $counter3 = new-object System.Diagnostics.CounterCreationData
    $counter3.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter3.CounterName = "Total Number of External OOFs Analyzed"
    [Void]$counterData.Add($counter3)

    $counter4 = new-object System.Diagnostics.CounterCreationData
    $counter4.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter4.CounterName = "Total Number of External OOFs Created"
    [Void]$counterData.Add($counter4)

    $counter5 = new-object System.Diagnostics.CounterCreationData
    $counter5.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter5.CounterName = "Total Number of AD Lookups Performed"
    [Void]$counterData.Add($counter5)

    $counter6 = new-object System.Diagnostics.CounterCreationData
    $counter6.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter6.CounterName = "Total Number of Internal Messages Routed Externally"
    [Void]$counterData.Add($counter6)

    $counter7 = new-object System.Diagnostics.CounterCreationData
    $counter7.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter7.CounterName = "Total Number of Messages Detected with X-Header"
    [Void]$counterData.Add($counter7)

    $counter8 = new-object System.Diagnostics.CounterCreationData
    $counter8.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter8.CounterName = "Total Number of Messages Bypassed Agent"
    [Void]$counterData.Add($counter8)

    $counter9 = new-object System.Diagnostics.CounterCreationData
    $counter9.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter9.CounterName = "Total Number of Subject Matched Messages Routed Externally"
    [Void]$counterData.Add($counter9)

	$counter10 = new-object System.Diagnostics.CounterCreationData
    $counter10.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter10.CounterName = "Total Number of Messages Detected with Subject-Routing X-Header"
    [Void]$counterData.Add($counter10)

	$counter11 = new-object System.Diagnostics.CounterCreationData
    $counter11.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter11.CounterName = "Total Number of Messages Detected with Sender-Based Routing X-Header"
    [Void]$counterData.Add($counter11)

	$counter12 = new-object System.Diagnostics.CounterCreationData
    $counter12.CounterType = [System.Diagnostics.PerformanceCounterType]::NumberOfItems32
    $counter12.CounterName = "Total Number of Sender Based Messages Routed Externally"
    [Void]$counterData.Add($counter12)

    [System.Diagnostics.PerformanceCounterCategory]::Create($categoryName, $categoryName, [System.Diagnostics.PerformanceCounterCategoryType]::SingleInstance, $counterData)
    [System.Diagnostics.PerformanceCounter]::CloseSharedResources()

}

###############################################################################
#
# Function Display-Eula
#
# PURPOSE
#    Writes Entry into Application Event Log
#
# INPUT
#
# RETURN
#    None.
#
###############################################################################
function Display-Eula
{
    # Load the Windows Forms assembly
    Add-Type -Assembly System.Windows.Forms

    $EulaText = Get-Content eula.txt

    # Create main form
    $form = New-Object Windows.Forms.Form
    $form.Text = "RickettsCorp Multi-Tenant Transport Agent EULA"
    $form.Size = New-Object Drawing.Size @(600,600)
    $form.FormBorderStyle = "Fixed3D"

    # Create the textbox
    $textbox = New-Object Windows.Forms.TextBox
    $textbox.Top = 5
    $textbox.Left = 5
    $textbox.Height = 520
    $textbox.multiline = $true
    $textBox.Width = 580
    $textbox.Anchor = "Left","Right"
    $textBox.Text = $EulaText
    $textbox.ReadOnly = $true
    $textbox.SelectionLength = 0
    $textbox.ScrollBars = "Both"

    # Create the button panel to hold the Accept and Decline buttons
    $buttonPanel = New-Object Windows.Forms.Panel
    $buttonPanel.Size = New-Object Drawing.Size @(600,40)
    $buttonPanel.Dock = "Bottom"

    # Create the Decline button
    $DeclineButton = New-Object Windows.Forms.Button
    $DeclineButton.Text = "Decline"
    $DeclineButton.DialogResult = "No"
    $DeclineButton.Top = $buttonPanel.Height - $DeclineButton.Height - 10
    $DeclineButton.Left = ($buttonPanel.Width / 2) + 15
    $DeclineButton.Anchor = "Right"

    # Create the Accept button
    $AcceptButton = New-Object Windows.Forms.Button
    $AcceptButton.Text = "Accept"
    $AcceptButton.DialogResult = "Yes"
    $AcceptButton.Top = $buttonPanel.Height - $AcceptButton.Height - 10
    $AcceptButton.Left = ($buttonPanel.Width / 2) - $AcceptButton.Width - 15
    $AcceptButton.Anchor = "Right"

    # Add the buttons to the button panel
    $buttonPanel.Controls.Add($AcceptButton)
    $buttonPanel.Controls.Add($DeclineButton)

    # Add the button panel and set the actions for the buttons
    $form.Controls.Add($buttonPanel)
    $form.Controls.Add($textbox)
    $form.AcceptButton = $AcceptButton
    $form.CancelButton = $DeclineButton
    $form.Add_Shown( { $form.Activate(); $DeclineButton.Focus() } )

    ## Show the form, and wait for the response
    $result = $form.ShowDialog()
    
    $result
}

###############################################################################
#
# Function Write-EventLogEntry
#
# PURPOSE
#    Writes Entry into Application Event Log
#
# INPUT
#    EventLogEntry = Text of event log entry
#    EventLogEntryType = Specifies the event type of an event log entry
#
# RETURN
#    None.
#
###############################################################################
function Write-EventLogEntry($EventLogEntry, $EventLogEntryType)
{
    $EventLogName = "Application"
    
    if(![System.Diagnostics.EventLog]::SourceExists($EventLogSource))
    {
        [System.Diagnostics.EventLog]::CreateEventSource($EventLogSource,$EventLogName)
    }

    Write-Output $EventLogEntry
    $EventLog = New-Object System.Diagnostics.Eventlog($EventLogName)
    $EventLog.Source = $EventLogSource
    $EventLog.WriteEntry($EventLogEntry, $EventLogEntryType, $EventLogID)
}

###############################################################################
#
# Function Get-ScriptPath
#
# PURPOSE
#    Returns full path current directory
#
# INPUT
#    None.
#
# RETURN
#    Full folder path
#
###############################################################################
function Get-ScriptPath
{
    Split-Path -Parent $MyInvocation.ScriptName
}

###############################################################################
#
# Function Load-XmlConfiguration
#
# PURPOSE
#    Load required Transport Agent configuration options from default XML file
#
# INPUT
#    None.
#
# RETURN
#    None.
#
###############################################################################
function Load-XmlConfiguration()
{
    $ScriptPath = Get-ScriptPath
    [XML]$Config = Get-Content "$ScriptPath\$ConfigFile"
    
    $Script:LogDirectory    = $Config.SelectSingleNode("//MultiTenantTransportAgent/Configuration/Logging/LogDirectory").InnerText
    $Script:EventLogSource  = $Config.SelectSingleNode("//MultiTenantTransportAgent/Configuration/Logging/EventLogSource").InnerText
    $Script:EventLogID      = $Config.SelectSingleNode("//MultiTenantTransportAgent/Configuration/Logging/EventLogId").InnerText
}

# Start script
. Main

###############################################################################
#
# Script End
#
###############################################################################